from django.contrib.auth.models import Group, User
from django.db import models


class Flag(models.Model):
    """A feature flag."""
    name = models.CharField(max_length=100, unique=True,
                            help_text='The human/computer readable name.')
    everyone = models.NullBooleanField(blank=True, help_text=(
        'Turn this flag on (True) or off (False) for all users.'))
    percent = models.DecimalField(max_digits=3, decimal_places=1, null=True,
                                  blank=True, help_text=(
        'A number between 0.0 and 99.9 to indicate a percentage of users for '
        'whom this flag will be active.'))
    superusers = models.BooleanField(default=True, help_text=(
        'Activate this flag for superusers?'))
    staff = models.BooleanField(default=False, help_text=(
        'Activate this flag for staff?'))
    authenticated = models.BooleanField(default=False, help_text=(
        'Activate this flag for authenticate users?'))
    groups = models.ManyToManyField(Group, blank=True, help_text=(
        'Activate this flag for these user groups.'))
    users = models.ManyToManyField(User, blank=True, help_text=(
        'Activate this flag for these users.'))
    rollout = models.BooleanField(default=False, help_text=(
        'Activate roll-out mode?'))

    def __unicode__(self):
        return self.name



from django import template

from waffle import is_active


register = template.Library()


class WaffleNode(template.Node):
    def __init__(self, nodelist_true, nodelist_false, flag_name):
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
        self.flag_name = flag_name

    def __repr__(self):
        return '<Waffle node: %s>' % self.flag_name

    def __iter__(self):
        for node in self.nodelist_true:
            yield node
        for node in self.nodelist_false:
            yield node

    def render(self, context):
        if is_active(context['request'], self.flag_name):
            return self.nodelist_true.render(context)
        return self.nodelist_false.render(context)


@register.tag
def waffle(parser, token):
    try:
        tag, flag_name = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, \
              "%r tag requires an argument" % token.contents.split()[0]

    flag_name = flag_name.strip('\'"')

    nodelist_true = parser.parse(('else', 'endwaffle'))
    token = parser.next_token()
    if token.contents == 'else':
        nodelist_false = parser.parse(('endwaffle',))
        parser.delete_first_token()
    else:
        nodelist_false = template.NodeList()

    return WaffleNode(nodelist_true, nodelist_false, flag_name)



from decimal import Decimal
import random

from django.conf import settings

from waffle.models import Flag


def is_active(request, flag_name):
    try:
        flag = Flag.objects.get(name=flag_name)
    except Flag.DoesNotExist:
        return getattr(settings, 'WAFFLE_DEFAULT', False)

    if getattr(settings, 'WAFFLE_OVERRIDE', False):
        if flag_name in request.GET:
            return request.GET[flag_name] == '1'

    if flag.everyone:
        return True
    elif flag.everyone == False:
        return False

    user = request.user

    if flag.authenticated and user.is_authenticated():
        return True

    if flag.staff and user.is_staff:
        return True

    if flag.superusers and user.is_superuser:
        return True

    if user in flag.users.all():
        return True

    for group in flag.groups.all():
        if group in user.groups.all():
            return True

    if flag.percent > 0:
        if not hasattr(request, 'waffles'):
            request.waffles = {}
        request.waffles[flag_name] = [False, flag.rollout]

        format = getattr(settings, 'WAFFLE_COOKIE', 'dwf_%s')
        cookie = format % flag_name
        if cookie in request.COOKIES:
            request.waffles[flag_name][0] = (request.COOKIES[cookie] == 'True')
            return request.waffles[flag_name][0]

        rand = Decimal(random.randint(0, 999)) / 10
        if rand <= flag.percent:
            request.waffles[flag_name][0] = True
            return True

    return False



from django.contrib import admin

from waffle.models import Flag


class FlagAdmin(admin.ModelAdmin):
    list_display = ('name', 'everyone', 'percent', 'superusers', 'staff',
                    'authenticated')
    list_filter = ('everyone', 'superusers', 'staff', 'authenticated')
    raw_id_fields = ('users', 'groups')


admin.site.register(Flag, FlagAdmin)



from django.conf import settings
from django.utils.encoding import smart_str


class WaffleMiddleware(object):
    def process_response(self, request, response):
        if hasattr(request, 'waffles'):
            secure = getattr(settings, 'WAFFLE_SECURE', False)
            max_age = getattr(settings, 'WAFFLE_MAX_AGE', 2592000)  # 1 month
            format = getattr(settings, 'WAFFLE_COOKIE', 'dwf_%s')
            for k in request.waffles:
                name = smart_str(format % k)
                active, rollout = request.waffles[k]
                if rollout and not active:
                    # "Inactive" is a session cookie during rollout mode.
                    age = None
                else:
                    age = max_age
                response.set_cookie(name, value=active, max_age=age,
                                    secure=secure)
        return response



from jingo import register
import jinja2

from waffle import is_active


@register.function
@jinja2.contextfunction
def waffle(context, flag_name):
    return is_active(context['request'], flag_name)
